/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*   Description:  an I/O library for reading a graph    						  */                                                   
/*                                                                           */
/*                                                                           */
/*   Authors:  Md. Mostofa Ali Patwary and Bharath Pattabiraman              */
/*            EECS Department, Northwestern University                       */
/*            email: {mpatwary,bpa342}@eecs.northwestern.edu                 */
/*                                                                           */
/*   Copyright, 2012, Md. Mostofa Ali Patwary and Bharath Pattabiraman       */
/*   See COPYRIGHT notice in top-level directory.                            */
/*                                                                           */
/*   Please site the following publication if you use this package           */
/*	  Bharath Pattabiraman, Md. Mostofa Ali Patwary, Assefaw H. Gebremedhin2, */
/*	  Wei-keng Liao, and Alok Choudhary.												  */
/*	  "A Hierarchical Pruning Algorithm for											  */ 
/*	  the Maximum Clique Problem on Large Graphs"									  */
/*   submitted to ALENEX'13 																  */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "GraphIO.h"

CGraphIO::~CGraphIO()
{
	m_vi_Vertices.clear();
	m_vi_OrderedVertices.clear();
	m_vi_Edges.clear();
	m_vd_Values.clear();
}

bool CGraphIO::readGraph(string s_InputFile, float connStrength)
{
	string fileExtension = getFileExtension(s_InputFile);		
	if(fileExtension == "mtx")
	{
		// matrix market format
		return ReadMatrixMarketAdjacencyGraph(s_InputFile, connStrength);
	}
	else if(fileExtension == "gr")
	{
		// gr format
		return ReadMeTiSAdjacencyGraph(s_InputFile);
	}
	else
		return false;	
}

bool CGraphIO::ReadMatrixMarketAdjacencyGraph(string s_InputFile, float connStrength)
{
	istringstream in2;
	string line="";
	map<int,vector<int> > nodeList;
	map<int,vector<double> > valueList;
	int col=0, row=0, rowIndex=0, colIndex=0;
	int entry_counter = 0, num_of_entries = 0;
	double value;

	ifstream in (s_InputFile.c_str());	
	if(!in) 
	{
		cout<<m_s_InputFile<<" not Found!"<<endl;
		return false;
	}

	char data[LINE_LENGTH];
	char banner[LINE_LENGTH];
	char mtx[LINE_LENGTH];
	char crd[LINE_LENGTH];
	char data_type[LINE_LENGTH];
	char storage_scheme[LINE_LENGTH];
	char* p;
	bool b_getValue = true;

	// read the banner
	getline(in, line);
	strcpy(data, line.c_str());

	if (sscanf(data, "%s %s %s %s %s", banner, mtx, crd, data_type, storage_scheme) != 5)
	{    
		cout << "Matrix file banner is missing!!!" << endl;
		return false;
	}

	// intersted about the forth part
	for (p=data_type; *p!='\0'; *p=tolower(*p),p++);

	if (strcmp(data_type, "pattern") == 0)
		b_getValue = false;

	getline(in, line);
	while(line.size()>0&&line[0]=='%') //ignore comment line
		getline(in,line);

	in2.str(line);
	in2 >> row >> col >> num_of_entries;

	if(row!=col) 
	{
		cout<<"* WARNING: GraphInputOutput::ReadMatrixMarketAdjacencyGraph()"<<endl;
		cout<<"*\t row!=col. This is not a square matrix. Can't process."<<endl;
		return false;
	}

	while(!in.eof() && entry_counter<num_of_entries) //there should be (num_of_entries+1) lines in the input file (excluding the comments)
	{
		getline(in,line);
		entry_counter++;

		//if(entry_counter % 1000000 == 0)
		//	cout << "reading line " << entry_counter << endl;

		if(line!="")
		{
			in2.clear();
			in2.str(line);

			in2 >> rowIndex >> colIndex >> value;
			rowIndex--;
			colIndex--;

			if(rowIndex < 0 || rowIndex >= row)
				cout << "Something wrong rowIndex " << rowIndex << " row " << row << endl;

			if(colIndex < 0 || colIndex >= col)
				cout << "Something wrong colIndex " << colIndex << " col " << col << endl;

			if(rowIndex == colIndex)
			{
				// ignore the self loop
				//cout << "self" << endl;
				continue;
			}

			if(rowIndex > colIndex) 
			{
				if(b_getValue)
				{
					if(value > connStrength)
					{
						nodeList[rowIndex].push_back(colIndex);
						nodeList[colIndex].push_back(rowIndex);
					}
				} 
				else 
				{
					nodeList[rowIndex].push_back(colIndex);
					nodeList[colIndex].push_back(rowIndex);
				}

				if(b_getValue && value > connStrength) 
				{
					valueList[rowIndex].push_back(value);
					valueList[colIndex].push_back(value);
				}
			}
			else
			{
				cout <<"* WARNING: GraphInputOutput::ReadMatrixMarketAdjacencyGraph()"<<endl;
				cout <<"\t Found a nonzero in the upper triangular. A symmetric Matrix Market file format should only specify the nonzeros in the lower triangular."<<endl;
				return false;
			}
		}
	}

	m_vi_Vertices.push_back(m_vi_Edges.size());

	for(int i=0;i < row; i++) 
	{
		m_vi_Edges.insert(m_vi_Edges.end(),nodeList[i].begin(),nodeList[i].end());
		m_vi_Vertices.push_back(m_vi_Edges.size());
	}

	if(b_getValue) 
	{
		for(int i=0;i<row; i++) 
		{
			m_vd_Values.insert(m_vd_Values.end(),valueList[i].begin(),valueList[i].end());
		}
	}

	//cout << "SIZE " << valueList.size() << endl;

	nodeList.clear();
	valueList.clear();
	CalculateVertexDegrees();
	return true;
}


bool CGraphIO::ReadMeTiSAdjacencyGraph(string s_InputFile)
{
	return true;
}

void CGraphIO::CalculateVertexDegrees()
{
		int i_VertexCount = m_vi_Vertices.size() - 1;

		m_i_MaximumVertexDegree =  -1;
		m_i_MinimumVertexDegree = -1;

		for(int i = 0; i < i_VertexCount; i++)
		{
			int i_VertexDegree = m_vi_Vertices[i + 1] - m_vi_Vertices[i];

			if(m_i_MaximumVertexDegree < i_VertexDegree)
			{
				m_i_MaximumVertexDegree = i_VertexDegree;
			}

			if(m_i_MinimumVertexDegree == -1)
			{
				m_i_MinimumVertexDegree = i_VertexDegree;
			}
			else if(m_i_MinimumVertexDegree > i_VertexDegree)
			{
				m_i_MinimumVertexDegree = i_VertexDegree;
			}
		}

		m_d_AverageVertexDegree = (double)m_vi_Edges.size()/i_VertexCount;

		return;
}

string CGraphIO::getFileExtension(string fileName)
{
	string::size_type result;
	string fileExtension = "";

	//1. see if the fileName is given in full path
	/*result = fileName.rfind("/", fileName.size() - 1);
	  if(result != string::npos)
	  {
	//found the path (file prefix)
	//get the path, including the last DIR_SEPARATOR
	path = fileName.substr(0,result+1);
	//remove the path from the fileName
	fileName = fileName.substr(result+1);
	}
	*/

	//2. see if the fileName has file extension. For example ".mtx"
	result = fileName.rfind('.', fileName.size() - 1);
	if(result != string::npos)
	{
		//found the fileExtension
		//get the fileExtension excluding the '.'
		fileExtension = fileName.substr(result+1);
		//remove the fileExtension from the fileName
		//fileName = fileName.substr(0,result);	
	}

	//3. get the name of the input file
	//name = fileName;

	return fileExtension;
}

void CGraphIO::ComputeSmallestLastOrdering()
{
	long long i, u, l;

    long long i_HighestInducedVertexDegree;

    long long i_VertexCount, i_InducedVertexDegree;

	long long i_VertexCountMinus1;

	long long i_InducedVertexDegreeCount;

    long long i_SelectedVertex, i_SelectedVertexCount;

	vector < long long > vi_InducedVertexDegree;

    vector< vector< long long > > vvi_GroupedInducedVertexDegree;

    vector< long long > vi_VertexLocation;

    i_VertexCount = STEP_DOWN(m_vi_Vertices.size());

    i_VertexCountMinus1 = i_VertexCount - 1; // = i_VertexCount - 1, used when inserting selected vertices into m_vi_OrderedVertices

    vi_InducedVertexDegree.clear();
	vi_InducedVertexDegree.reserve(i_VertexCount);

    vvi_GroupedInducedVertexDegree.clear();
    vvi_GroupedInducedVertexDegree.resize(i_VertexCount);

    vi_VertexLocation.clear();
	vi_VertexLocation.reserve(i_VertexCount);

    i_SelectedVertex = _UNKNOWN;

    i_HighestInducedVertexDegree = _FALSE;


	for(i=0; i<i_VertexCount; i++)
	{
		//get vertex degree for each vertex
		i_InducedVertexDegree = m_vi_Vertices[STEP_UP(i)] - m_vi_Vertices[i];

		//vi_InducedVertexDegree[i] = vertex degree of vertex i
		vi_InducedVertexDegree.push_back(i_InducedVertexDegree);

		// vector vvi_GroupedInducedVertexDegree[i] = all the vertices with degree i
		// for every new vertex with degree i, it will be pushed to the back of vector vvi_GroupedInducedVertexDegree[i]
		vvi_GroupedInducedVertexDegree[i_InducedVertexDegree].push_back(i);
			
		//vi_VertexLocation[i] = location of vertex i in vvi_GroupedInducedVertexDegree[i_InducedVertexDegree]
		vi_VertexLocation.push_back(vvi_GroupedInducedVertexDegree[i_InducedVertexDegree].size() - 1);

		//get max degree (i_HighestInducedVertexDegree)
		if(i_HighestInducedVertexDegree < i_InducedVertexDegree)
        {
       		i_HighestInducedVertexDegree = i_InducedVertexDegree;
        }
	}

	m_vi_OrderedVertices.clear();
    m_vi_OrderedVertices.resize(i_VertexCount, _UNKNOWN);

	i_SelectedVertexCount = _FALSE;
	long long iMin = 1;

	// just counting the number of vertices that we have worked with,
	// stop when i_SelectedVertexCount == i_VertexCount, i.e. we have looked through all the vertices
	while(i_SelectedVertexCount < i_VertexCount)
   	{	
		if(iMin != 0 && vvi_GroupedInducedVertexDegree[iMin - 1].size() != _FALSE)
			iMin--;

		//pick the vertex with smallest degree
		for(i=iMin; i<STEP_UP(i_HighestInducedVertexDegree); i++)
       	{
       		i_InducedVertexDegreeCount = vvi_GroupedInducedVertexDegree[i].size();

           	if(i_InducedVertexDegreeCount != _FALSE)
          	{
           		i_SelectedVertex = vvi_GroupedInducedVertexDegree[i].back();
				//remove the i_SelectedVertex from vvi_GroupedInducedVertexDegree
				vvi_GroupedInducedVertexDegree[i].pop_back();
           		break;
            }
			else
				iMin++;
		}
		// and vi_VertexLocation
		for(i=m_vi_Vertices[i_SelectedVertex]; i<m_vi_Vertices[STEP_UP(i_SelectedVertex)]; i++)
        {
        	u = m_vi_Edges[i];

            if(vi_InducedVertexDegree[u] == _UNKNOWN)
            {
            	continue;
            }
			// move the last element in this bucket to u's position to get rid of expensive erase operation
           	if(vvi_GroupedInducedVertexDegree[vi_InducedVertexDegree[u]].size() > 1)
           	{
           		l = vvi_GroupedInducedVertexDegree[vi_InducedVertexDegree[u]].back();

               	vvi_GroupedInducedVertexDegree[vi_InducedVertexDegree[u]][vi_VertexLocation[u]] = l;


               	vi_VertexLocation[l] = vi_VertexLocation[u];
            }
			// remove last element from this bucket
        	vvi_GroupedInducedVertexDegree[vi_InducedVertexDegree[u]].pop_back();

			// reduce degree of u by 1
    	    vi_InducedVertexDegree[u]--;

			// move u to appropriate bucket
    	    vvi_GroupedInducedVertexDegree[vi_InducedVertexDegree[u]].push_back(u);

			// update vi_VertexLocation[u] since it has now been changed
    	    vi_VertexLocation[u] = vvi_GroupedInducedVertexDegree[vi_InducedVertexDegree[u]].size() - 1;
        }
			
		//Mark the i_SelectedVertex as read, so that we don't look at it again
	    vi_InducedVertexDegree[i_SelectedVertex] = _UNKNOWN;
    	// insert i_SelectedVertex into m_vi_OrderedVertices
	    m_vi_OrderedVertices[i_VertexCountMinus1 - i_SelectedVertexCount] = i_SelectedVertex;
		//increment i_SelectedVertexCount
   		i_SelectedVertexCount = STEP_UP(i_SelectedVertexCount);
	}
		
	// clear the buffer
   	vi_InducedVertexDegree.clear();
   	vi_VertexLocation.clear();
   	vvi_GroupedInducedVertexDegree.clear();	
}

int CGraphIO::MaximumBackDegree()
{
	//create the map from vertexID to orderingID
	vector<long long> vectorID2orderingID;
	vectorID2orderingID.resize(m_vi_OrderedVertices.size(),-1);
	for(long long i=0; i < m_vi_OrderedVertices.size(); i++) {
		vectorID2orderingID[m_vi_OrderedVertices[i]] = i;
	}

	//double check
	for(long long i=0; i < vectorID2orderingID.size(); i++) {
		if(vectorID2orderingID[i]==-1) {
			cerr<<"What the hell? There is a vertex missing"<<endl;
		}
	}

	//Now, for each vertex, find its MaxBackDegree
	long long i_MaxBackDegree = -1;
	long long i_CurrentVertexBackDegre = -1;
	long long currentOrderingID = -1;
	for(long long i=0; i < m_vi_Vertices.size() - 1; i++) {
		currentOrderingID = vectorID2orderingID[i];
		i_CurrentVertexBackDegre = 0;
		//for through all the D1 neighbor of that vertex
		for(long long j = m_vi_Vertices[i]; j < m_vi_Vertices[i + 1]; j++) {
			if(vectorID2orderingID[m_vi_Edges[j]] < currentOrderingID) i_CurrentVertexBackDegre++;
		}
		if( i_MaxBackDegree < i_CurrentVertexBackDegre) i_MaxBackDegree = i_CurrentVertexBackDegre;
	}

	return i_MaxBackDegree;	
}
